-- Digital DAC  kner 2011
--
-- ein digitaler Eingang 8bit -128 .. + 127 wird in einen 1-Bit Datenstrom gewandelt
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;
use IEEE.std_logic_arith.all;
entity top is
    Port ( a : in  STD_LOGIC_VECTOR (7 downto 0);
			  clk: in STD_LOGIC; 
			  sclr: in STD_LOGIC; 
           s : out  STD_LOGIC;  
			  heartbeat: out STD_LOGIC; 
			  control_led:out STD_LOGIC 
			 -- xout: out std_logic_vector(2 downto 0)
		);
end top;

architecture Behavioral of top is
   COMPONENT ddc
	PORT(
		input : IN std_logic_vector(9 downto 0);          
		msb : OUT std_logic;
		vref : OUT std_logic_vector(7 downto 0)
		);
	END COMPONENT;
	COMPONENT integrator
	  PORT (
		 b : IN STD_LOGIC_VECTOR(8 DOWNTO 0);
		 clk : IN STD_LOGIC;
		 sclr : IN STD_LOGIC; 
		 q : OUT STD_LOGIC_VECTOR(9 DOWNTO 0)
	  );
	END COMPONENT;
	COMPONENT add_sub
	  PORT (
		 a : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 b : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 clk : IN STD_LOGIC;
		 s : OUT STD_LOGIC_VECTOR(8 DOWNTO 0)
	  );
	END COMPONENT;
	COMPONENT one_hundred_K_counter
	PORT(
		clk : IN std_logic;
		rst : IN std_logic;
		cken : IN std_logic;          
		dout_fast : OUT std_logic;
		dout_100k : OUT std_logic
		);
	END COMPONENT;

	signal x: STD_LOGIC_VECTOR(8 DOWNTO 0);
	signal y: STD_LOGIC_VECTOR(7 DOWNTO 0);
	signal s1: STD_LOGIC_VECTOR(9 DOWNTO 0);
	signal rst: STD_LOGIC := '0'; 
	signal cken: STD_LOGIC := '1'; 
	signal dout_fast: STD_LOGIC; 
	signal dout_100k : std_logic; 
   signal msb : std_logic; 
	signal y1 : std_logic_vector(7 downto 0); 
	
begin
  adder1 : add_sub
  PORT MAP (
    a => a,
    b => y,
    clk => dout_100k,
    s => x
  );
 
 int1 : integrator
  PORT MAP (
    b => x,
    clk => dout_100k,
	 sclr => sclr,
    q => s1
  );
  
  Inst_ddc: ddc PORT MAP(
		input => s1,
		msb => msb,
		vref => y
	);

  Inst_one_hundred_K_counter: one_hundred_K_counter PORT MAP(
		clk => clk,
		rst => rst,
		cken => cken,
		dout_fast => dout_fast,
		dout_100k => dout_100k
	);
	rst <= '1';
   cken <= '1'; 	
  -- 1Bit DDC (digital to digital converter)
  -- ich verwende einen Addierer am Eingang, daher brauche ich eine 
  -- vorzeichenbehaftete Rckkopplung 
  s <= msb; 
  heartbeat <= dout_100k; 
  control_led <= a(0);
  end Behavioral;

